@@ -59,7 +59,7 @@ class DiagnosticMessage(object):
         else:
             name = "NULL"
 
-        result += "        addRecurringRequest(diagnosticsManager, &getCanBuses()[%d], &request, %s, %s, %s, %s, %f);\n        }\n" % (
+        result += " " * 8 + "addRecurringRequest(diagnosticsManager, &getCanBuses()[%d], &request, %s, %s, %s, %s, %f);\n        }\n" % (
                 self.message_set.lookup_bus_index(self.bus_name),
                 name,
                 str(self.multiple_responses).lower(),
@@ -230,6 +230,7 @@ class CanBus(object):
             default_raw_can_mode=None,
             raw_can_mode=None,
             raw_writable=False,
+            loopback=False,
             **kwargs):
         self.name = name
         self.speed = speed
@@ -240,6 +241,7 @@ class CanBus(object):
             self.max_message_frequency = default_max_message_frequency
         self.raw_can_mode = raw_can_mode or default_raw_can_mode
         self.raw_writable = raw_writable
+        self.loopback = loopback
 
     def valid(self):
         return self.controller in self.VALID_BUS_ADDRESSES
@@ -265,15 +267,29 @@ class CanBus(object):
     def add_message(self, message):
         self.messages.append(message)
 
+    @property
+    def passthrough(self):
+        return self.raw_can_mode != "off"
+
+    @property
+    def bypass_filters(self):
+        return self.raw_can_mode == "unfiltered"
+
     def __str__(self):
         result = """        {{ speed: {bus_speed},
         address: {controller},
         maxMessageFrequency: {max_message_frequency},
-        rawWritable: {raw_writable}
+        rawWritable: {raw_writable},
+        passthroughCanMessages: {passthrough},
+        bypassFilters: {bypass_filters},
+        loopback: {loopback}
         }},"""
         return result.format(bus_speed=self.speed, controller=self.controller,
                 max_message_frequency=self.max_message_frequency,
-                raw_writable=str(self.raw_writable).lower())
+                raw_writable=str(self.raw_writable).lower(),
+                passthrough=str(self.passthrough).lower(),
+                bypass_filters=str(self.bypass_filters).lower(),
+                loopback=str(self.loopback).lower())
 
 
 class ConfigurationError(Exception):
@@ -312,7 +328,7 @@ class Signal(object):
     def merge_signal(self, data):
         self.name = data.get('name', self.name)
         self.enabled = data.get('enabled', self.enabled)
-        self.generic_name = data.get('generic_name', self.generic_name)
+        self.generic_name = data.get('generic_name', self.name)
         self.bit_position = data.get('bit_position', self.bit_position)
         self.bit_size = data.get('bit_size', self.bit_size)
         self.factor = data.get('factor', self.factor)
