@@ -3,18 +3,20 @@ from __future__ import absolute_import
 
 import logging
 import socket
-from .base import BytestreamDataSource, DataSourceError
+
+from .base import DataSourceError
+from .socket import SocketDataSource
 
 LOG = logging.getLogger(__name__)
 
 
-class NetworkDataSource(BytestreamDataSource):
+class NetworkDataSource(SocketDataSource):
     """A data source reading from a network socket, as implemented
     in the openxc-vehicle-simulator .
     """
     DEFAULT_PORT = 50001
 
-    def __init__(self, callback=None, host=None, port=None, log_mode=None):
+    def __init__(self, host=None, port=None, **kwargs):
         """Initialize a connection to the network socket.
 
         Kwargs:
@@ -25,7 +27,7 @@ class NetworkDataSource(BytestreamDataSource):
         Raises:
             DataSourceError if the socket connection cannot be opened.
         """
-        super(NetworkDataSource, self).__init__(callback, log_mode)
+        super(NetworkDataSource, self).__init__(**kwargs)
         self.host = host or socket.gethostbyname(socket.gethostname())
         self.port = port or self.DEFAULT_PORT
         self.port = int(self.port)
@@ -38,17 +40,3 @@ class NetworkDataSource(BytestreamDataSource):
                     "%s:%s: %s" % (self.host,self.port, e))
         else:
             LOG.debug("Opened socket connection at %s:%s", self.host, self.port)
-
-    def read(self):
-        try:
-            line = ""
-            while '\x00' not in line:
-                # TODO this is fairly inefficient
-                line += self.socket.recv(1)
-        except (OSError, socket.error) as e:
-            raise DataSourceError("Unable to read from socket connection at  "
-                    "%s:%s: %s" % (self.host,self.port, e))
-        if not line:
-            raise DataSourceError("Unable to read from socket connection at  "
-                "%s:%s" % (self.host,self.port))
-        return line
