@@ -3,6 +3,7 @@ from __future__ import absolute_import
 
 import logging
 import usb.core
+import usb.util
 
 from .base import BytestreamDataSource, DataSourceError
 
@@ -14,14 +15,19 @@ class UsbDataSource(BytestreamDataSource):
     DEFAULT_VENDOR_ID = 0x1bc4
     DEFAULT_PRODUCT_ID = 0x0001
     DEFAULT_READ_REQUEST_SIZE = 512
-    DEFAULT_READ_TIMEOUT = 1000000
+
+    # If we don't get DEFAULT_READ_REQUEST_SIZE bytes within this number of
+    # milliseconds, bail early and return whatever we have - could be zero,
+    # could be just less than 512. If data is really pumpin' we can get better
+    # throughput if the READ_REQUEST_SIZE is higher, but this delay has to be
+    # low enough that a single request isn't held back too long.
+    DEFAULT_READ_TIMEOUT = 200
 
     DEFAULT_INTERFACE_NUMBER = 0
-    TRANSLATED_IN_ENDPOINT = 2
+    VEHICLE_DATA_IN_ENDPOINT = 2
     LOG_IN_ENDPOINT = 11
 
-    def __init__(self, callback=None, vendor_id=None, product_id=None,
-            log_mode=None):
+    def __init__(self, vendor_id=None, product_id=None, **kwargs):
         """Initialize a connection to the USB device's IN endpoint.
 
         Kwargs:
@@ -40,7 +46,7 @@ class UsbDataSource(BytestreamDataSource):
             DataSourceError if the USB device with the given vendor ID is not
             connected.
         """
-        super(UsbDataSource, self).__init__(callback, log_mode)
+        super(UsbDataSource, self).__init__(**kwargs)
         if vendor_id is not None and not isinstance(vendor_id, int):
             vendor_id = int(vendor_id, 0)
         self.vendor_id = vendor_id or self.DEFAULT_VENDOR_ID
@@ -63,11 +69,15 @@ class UsbDataSource(BytestreamDataSource):
         raise DataSourceError("No USB vehicle interface detected - is one plugged in?")
 
     def read(self, timeout=None):
-        return self._read(self.TRANSLATED_IN_ENDPOINT, timeout)
+        return self._read(self.VEHICLE_DATA_IN_ENDPOINT, timeout)
 
     def read_logs(self, timeout=None):
         return self._read(self.LOG_IN_ENDPOINT, timeout, 64)
 
+    def stop(self):
+        super(UsbDataSource, self).stop()
+        usb.util.dispose_resources(self.device)
+
     def _read(self, endpoint_address, timeout=None,
             read_size=DEFAULT_READ_REQUEST_SIZE):
         timeout = timeout or self.DEFAULT_READ_TIMEOUT
@@ -76,4 +86,7 @@ class UsbDataSource(BytestreamDataSource):
                     read_size, self.DEFAULT_INTERFACE_NUMBER, timeout
                     ).tostring()
         except (usb.core.USBError, AttributeError) as e:
+            if e.errno == 110:
+                # Timeout, it may just not be sending
+                return ""
             raise DataSourceError("USB device couldn't be read", e)
