@@ -21,6 +21,21 @@ def version(interface):
 def device_id(interface):
     print("Device ID is %s" % interface.device_id())
 
+def passthrough(interface, bus, passthrough_enabled):
+    if interface.set_passthrough(bus, passthrough_enabled):
+        print("Bus %u passthrough set to %s" % (bus, passthrough_enabled))
+
+def af_bypass(interface, bus, bypass):
+    if interface.set_acceptance_filter_bypass(bus, bypass):
+        if bypass:
+            bypass_string = "bypassed"
+        else:
+            bypass_string = "enabled"
+        print("Bus %u AF is now %s" % (bus, bypass_string))
+
+def set_payload_format(interface, payload_format):
+    if interface.set_payload_format(payload_format):
+        print("Changed payload format to %s" % payload_format)
 
 def write_file(interface, filename):
     first_timestamp = None
@@ -60,25 +75,38 @@ def parse_options():
     parser = argparse.ArgumentParser(description="Send control messages to an "
             "attached OpenXC vehicle interface", parents=[device_options()])
     parser.add_argument("command", type=str,
-            choices=['version', 'write', 'id'])
+            choices=['version', 'write', 'id', 'set'])
     write_group = parser.add_mutually_exclusive_group()
     write_group.add_argument("--name", action="store", dest="write_name",
             help="name for message write request")
     write_group.add_argument("--id", action="store", dest="write_id",
             help="ID for raw message write request")
-    parser.add_argument("--bus", action="store", dest="write_bus",
+    parser.add_argument("--bus", action="store", dest="bus",
             default=1,
-            help="bus number for raw message write request")
+            help="CAN bus number for the control request")
     parser.add_argument("--value", action="store", dest="write_value",
             help="optional value for message write request")
     parser.add_argument("--event", action="store", dest="write_event",
             help="optional event for message write request")
     parser.add_argument("--data", action="store", dest="write_data",
             help="data for raw message write request")
+    parser.add_argument("--frame-format", action="store",
+            dest="write_frame_format",  choices=['standard', 'extended'],
+            help="explicit frame format for raw message write request")
     write_group.add_argument("-f", "--write-input-file", action="store",
             dest="write_input_file",
             help="the path to a file with a list of raw or translated "
                     "messages to write to the selected vehicle interface")
+    parser.add_argument("--passthrough", action="store_true", default=None,
+            dest="passthrough_enabled")
+    parser.add_argument("--no-passthrough", action="store_false", default=None,
+            dest="passthrough_enabled")
+    parser.add_argument("--af-bypass", action="store_true", default=None,
+            dest="af_bypass")
+    parser.add_argument("--no-af-bypass", action="store_false", default=None,
+            dest="af_bypass")
+    parser.add_argument("--new-payload-format", action="store", default=None,
+            choices=['json', 'protobuf'], dest="new_payload_format")
 
     return parser.parse_args()
 
@@ -91,13 +119,17 @@ def main():
     interface = interface_class(**interface_kwargs)
     interface.start()
 
-    # wait for receiver thread to spool up and connect
-    time.sleep(.5)
-
     if arguments.command == "version":
         version(interface)
     elif arguments.command == "id":
         device_id(interface)
+    elif arguments.command == "set":
+        if arguments.passthrough_enabled is not None:
+            passthrough(interface, int(arguments.bus), arguments.passthrough_enabled)
+        if arguments.af_bypass is not None:
+            af_bypass(interface, int(arguments.bus), arguments.af_bypass)
+        if arguments.new_payload_format is not None:
+            set_payload_format(interface, arguments.new_payload_format)
     elif arguments.command.startswith("write"):
         if arguments.command == "write":
             if arguments.write_name:
@@ -107,15 +139,16 @@ def main():
             elif arguments.write_id:
                 if not arguments.write_data:
                     sys.exit("%s requires an id and data" % arguments.command)
-                interface.write(bus=int(arguments.write_bus),
+                # TODO we should use unhexlify as with the diagnostic command
+                # payloads so we can standardize the API and not deal with hex
+                # strings in code
+                interface.write(bus=int(arguments.bus),
                         id=arguments.write_id,
-                        data=arguments.write_data)
+                        data=arguments.write_data,
+                        frame_format=arguments.write_frame_format)
             elif arguments.write_input_file:
                 write_file(interface, arguments.write_input_file)
             else:
                 sys.exit("%s requires a signal name, message ID or filename" % arguments.command)
     else:
         print("Unrecognized command \"%s\"" % arguments.command)
-
-    # TODO need a better way to wait for log messages after writing
-    time.sleep(1)
