@@ -10,7 +10,7 @@ units.predefined.define_units()
 
 Percentage = unit('%')
 Meter = unit('m')
-Kilometer = scaled_unit('m', 'km', 1000)
+Kilometer = scaled_unit('km', 'm', 1000)
 Hour = unit('h')
 KilometersPerHour = unit('km') / unit('h')
 RotationsPerMinute = unit('rotations') / unit('m')
